@@ -734,14 +734,26 @@ sub _do_show {
     # XXX TODO, we should search all component roots
 
     if ($path !~ m{/$}
-        and -d Jifty::Util->absolute_path( Jifty->config->framework('Web')->{'TemplateRoot'} . $path))
+        and -d Jifty::Util->absolute_path(
+            Jifty->config->framework('Web')->{'TemplateRoot'} . $path
+        )
+        )
     {
         $self->_do_show( $path . "/" );
     }
 
-    # Set the request path
-    request->path($path);
-    $self->render_template(request->path);
+    my $abs_template_path = Jifty::Util->absolute_path(
+        Jifty->config->framework('Web')->{'TemplateRoot'} . $path );
+    my $abs_root_path = Jifty::Util->absolute_path(
+        Jifty->config->framework('Web')->{'TemplateRoot'} );
+
+    if ( $abs_template_path !~ /^\Q$abs_root_path\E/ ) {
+        request->path('/__jifty/errors/500');
+    } else {
+        # Set the request path
+        request->path($path);
+    }
+    $self->render_template( request->path );
 
     last_rule;
 }
@@ -780,7 +792,8 @@ Once it's done with that, it runs all the cleanup rules defined with C<after>.
 sub _do_dispatch {
     my $self = shift;
 
-    $self->{path} = shift;
+    # Requests should always start with a leading /
+    $self->{path} = "/".shift;
     $self->{cwd}  = '';
 
     # Normalize the path.
@@ -1190,4 +1203,59 @@ sub _match_deferred {
     return @matches;
 }
 
+=head2 dump_rules
+
+Dump all defined rules in debug log. It will be called by Jifty on startup.
+
+=cut
+
+sub dump_rules {
+    my $self = shift;
+
+    no strict 'refs';
+    foreach my $stage ( qw/SETUP RUN CLEANUP/ ) {
+
+        my $log = '';
+        foreach my $r ( @{ $self . '::RULES_' . $stage } ) {
+            $log .= _unroll_dumpable_rules( 0,$r );
+        }
+
+        Jifty->log->debug( "Rules in stage $stage:\n", $log) if ($log);
+    }
+};
+
+=head2 _unroll_dumpable_rules LEVEL,RULE
+
+Walk all rules defined in dispatcher starting at rule
+C<RULE> and indentation level C<LEVEL>
+
+=cut
+
+sub _unroll_dumpable_rules {
+    my ($level, $rule) = @_;
+    my $log =
+        # indentation
+        ( "    " x $level ) .
+        # op
+        ( $rule->[0] || "undef op" ) . ' ' .
+        # arguments
+        (
+            ! defined( $rule->[1] )   ? ""                                          :
+            ref $rule->[1] eq 'ARRAY' ? "'" . join("','", @{ $rule->[1] }) . "'" :
+            ref $rule->[1] eq 'HASH'  ? $rule->[1]->{method} . " '" . $rule->[1]->{""} ."'" :
+            ref $rule->[1] eq 'CODE'  ? '{...}' :
+                                        "'" . $rule->[1] . "'"
+        ) .
+        "\n";
+
+    if (ref $rule->[2] eq 'ARRAY') {
+        $level++;
+        foreach my $sr ( @{ $rule->[2] } ) {
+            $log .= _unroll_dumpable_rules( $level, $sr );
+        }
+    }
+
+    return $log;
+}
+
 1;
