@@ -3,15 +3,19 @@ use strict;
 
 =head1 NAME
 
-Jifty::Plugin::Login::Action::SendAccountConfirmation
+Jifty::Plugin::Login::Action::ResendConfirmation
 
 =cut
 
-package Jifty::Plugin::Login::Action::SendAccountConfirmation;
-use base qw/Jifty::Action Jifty::Plugin::Login/;
+package Jifty::Plugin::Login::Action::ResendConfirmation;
+use base qw/Jifty::Action/;
+my $LoginUser   = $Jifty::Plugin::Login::LoginUserClass;
+my $CurrentUser = $Jifty::Plugin::Login::CurrentUserClass;
 
 __PACKAGE__->mk_accessors(qw(user_object));
 
+use Jifty::Plugin::Login::Model::User;
+
 =head2 arguments
 
 The field for C<ResendConfirmation> is:
@@ -28,7 +32,7 @@ sub arguments {
     return (
         {
             address => {
-                label         => _('email address'),
+                label         => 'email address',
                 mandatory     => 1,
                 default_value => "",
             },
@@ -44,8 +48,6 @@ Create an empty user object to work with
 
 sub setup {
     my $self = shift;
-    my $LoginUser = $self->LoginUserClass();
-    my $CurrentUser = $self->CurrentUserClass();
 
     $self->user_object(
         $LoginUser->new( current_user => $CurrentUser->superuser ) );
@@ -60,22 +62,20 @@ Make sure their email address is an unconfirmed user.
 sub validate_address {
     my $self  = shift;
     my $email = shift;
-    my $LoginUser = $self->LoginUserClass();
-    my $CurrentUser = $self->CurrentUserClass();
 
     return $self->validation_error(
-        address => _("That doesn't look like an email address.") )
+        address => "That doesn't look like an email address." )
       unless ( $email =~ /\S\@\S/ );
 
     $self->user_object(
         $LoginUser->new( current_user => $CurrentUser->superuser ) );
     $self->user_object->load_by_cols( email => $email );
     return $self->validation_error(
-        address => _("It doesn't look like there's an account by that name.") )
+        address => "It doesn't look like there's an account by that name." )
       unless ( $self->user_object->id );
 
     return $self->validation_error(
-        address => _("It looks like you're already confirmed.") )
+        address => "It looks like you're already confirmed." )
       if ( $self->user_object->email_confirmed );
 
     return $self->validation_ok('address');
@@ -91,7 +91,7 @@ sub take_action {
     my $self = shift;
     Jifty::Plugin::Login::Notification::ConfirmAddress->new(
         to => $self->user_object )->send;
-    return $self->result->message(_("Confirmation resent."));
+    return $self->result->message("Confirmation resent.");
 }
 
 1;
