@@ -3,13 +3,12 @@ use strict;
 
 =head1 NAME
 
-Jifty::Plugin::Login::Action::SendPasswordReminder
+Jifty::Plugin::Login::Action::SendLostPasswordConfirmation
 
 =cut
 
-package Jifty::Plugin::Login::Action::SendPasswordReminder;
-use base qw/Jifty::Action Jifty::Plugin::Login/;
-
+package Jifty::Plugin::Login::Action::SendLostPasswordConfirmation;
+use base qw/Jifty::Plugin::Login::Action Jifty::Action Jifty::Plugin::Login/;
 
 __PACKAGE__->mk_accessors(qw(user_object));
 
@@ -17,7 +16,7 @@ use Jifty::Plugin::Login::Model::User;
 
 =head2 arguments
 
-The field for C<SendLostPasswordReminder> is:
+The field for C<SendLostPasswordConfirmation> is:
 
 =over 4
 
@@ -31,7 +30,7 @@ sub arguments {
     return (
         {
             address => {
-                label     => _('email address'),
+                label     => 'email address',
                 mandatory => 1,
             },
         }
@@ -68,14 +67,14 @@ sub validate_address {
     my $CurrentUser = $self->CurrentUserClass();
 
     return $self->validation_error(
-        address => _("That doesn't look like an email address.") )
+        address => "That doesn't look like an email address." )
       unless ( $email =~ /\S\@\S/ );
 
     $self->user_object(
         $LoginUser->new( current_user => $CurrentUser->superuser ) );
     $self->user_object->load_by_cols( email => $email );
     return $self->validation_error(
-        address => _("It doesn't look like there's an account by that name.") )
+        address => "It doesn't look like there's an account by that name." )
       unless ( $self->user_object->id );
 
     return $self->validation_ok('address');
@@ -83,7 +82,7 @@ sub validate_address {
 
 =head2 take_action
 
-Send out a Reminder email giving a link to a password-reset form.
+Send out a confirmation email giving a link to a password-reset form.
 
 =cut
 
@@ -92,7 +91,7 @@ sub take_action {
     Jifty::Plugin::Login::Notification::ConfirmLostPassword->new(
         to => $self->user_object )->send;
     return $self->result->message(
-        _("A link to reset your password has been sent to your email account."));
+        "A link to reset your password has been sent to your email account.");
 }
 
 1;
