@@ -30,16 +30,19 @@ Jifty.ContextMenu = {
     hide: function(id) {
         var ul = document.getElementById(id);
 
-        var li = Jifty.ContextMenu.getParentListItem(ul);
-        Element.removeClassName(li, "open");
-        Element.addClassName(li, "closed");
-                             
-        ul.style.display = "none";
+        if ( ul ) {
+            var li = Jifty.ContextMenu.getParentListItem(ul);
+            Element.removeClassName(li, "open");
+            Element.addClassName(li, "closed");
+            
+            ul.style.display = "none";
+        }
         Jifty.ContextMenu.currently_open = "";
     },
 
     show: function(id) {
         var ul = document.getElementById(id);
+        if ( !ul ) return;
         
         if (   Jifty.ContextMenu.currently_open
             && Jifty.ContextMenu.currently_open != ul.id )
@@ -49,21 +52,32 @@ Jifty.ContextMenu = {
     
         var li = Jifty.ContextMenu.getParentListItem(ul);
 
-        if ( ul.style.position == "" ) {
-            var x = Jifty.Utils.findPosX( li );
-            var y = Jifty.Utils.findPosY( li ) + li.offsetHeight;
-
-            ul.style.position = "absolute";
-            ul.style.left     = x + "px";
-            ul.style.top      = y + "px";
-            ul.style.width    = li.offsetWidth * 2 + "px";
+        ul.style.position = "absolute";
+        ul.style.width    = li.offsetWidth * 2 + "px";
+        
+        /* Use position: relative based positioning for every browser
+           but IE, which needs to use absolute positioning */
+        if ( Jifty.Utils.isMSIE ) {
+            if ( ul.style.position == "" ) {
+                var x = Jifty.Utils.findPosX( li );
+                var y = Jifty.Utils.findPosY( li ) + li.offsetHeight;
+                
+                ul.style.left = x + "px";
+                ul.style.top  = y + "px";
+            }
+        }
+        else {
+            ul.style.left     = "-1px";
+            ul.style.top      = li.offsetHeight - 2 + "px";
+            li.style.position = "relative";
         }
 
         Element.removeClassName(li, "closed");
         Element.addClassName(li, "open");
-         
+        
         ul.style.display = "block";
         Jifty.ContextMenu.currently_open = ul.id;
+        Jifty.ContextMenu.scrollToShow( ul.id );
     },
 
     hideOpenMenu: function(event) {
@@ -84,6 +98,17 @@ Jifty.ContextMenu = {
         if (Jifty.ContextMenu.currently_open) {
             Jifty.ContextMenu.hide(Jifty.ContextMenu.currently_open);
         }
+    },
+
+    scrollToShow: function(id) {
+        var ul        = $(id);
+        var y         = Jifty.Utils.findPosY( ul ) + ul.offsetHeight + 10;
+        var scrollTop = Jifty.Utils.getScrollTop();
+        var screen    = Jifty.Utils.findScreenHeight() + scrollTop;
+        var diff      = y - screen;
+        
+        if ( diff > 0 )
+             Jifty.SmoothScroll.scrollTo( scrollTop + diff );
     }
 };
 
